﻿Option Strict On
Imports Microsoft.Kinect

Class MainWindow
    Dim Kinect As KinectSensor
    Dim mySkeleton As Skeleton
    Dim myScreenImageRect As Int32Rect
    Dim myDepthPixelData As Short()
    Dim myColorPixelData As Byte()
    Dim rightZ As Single
    Dim leftZ As Single
    Dim headZ As Single
    Dim myDepth As Double

    Property HumanImage As WriteableBitmap

    Private Sub MainWindow_Loaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles Me.Loaded

        myDepth = 1000
        init_kinect()

        AddHandler CompositionTarget.Rendering, Sub(renderSender As Object, renderArgs As EventArgs)
                                                    Using colorFrame As ColorImageFrame = Kinect.ColorStream.OpenNextFrame(100)
                                                        Using depthFrame As DepthImageFrame = Kinect.DepthStream.OpenNextFrame(100)
                                                            RenderScreen(colorFrame, depthFrame)
                                                        End Using
                                                    End Using

                                                End Sub
        DataContext = Me
    End Sub

    Private Sub init_kinect()
        Try
            If KinectSensor.KinectSensors.Count = 0 Then
                MessageBox.Show("KINECTが接続されておりません。")
                Exit Sub
            Else
                Kinect = KinectSensor.KinectSensors(0)
            End If
            Dim depthStream As DepthImageStream = Kinect.DepthStream
            Kinect.DepthStream.Enable(DepthImageFormat.Resolution640x480Fps30)
            Kinect.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30)
            Kinect.SkeletonStream.Enable()

            AddHandler Kinect.AllFramesReady, AddressOf kinect_AllFramesReady
            myDepthPixelData = New Short(Kinect.DepthStream.FramePixelDataLength - 1) {}
            myColorPixelData = New Byte(Kinect.ColorStream.FramePixelDataLength - 1) {}
            HumanImage = New WriteableBitmap(depthStream.FrameWidth, depthStream.FrameHeight, 96, 96, PixelFormats.Bgr32, Nothing) ' New WriteableBitmap(640, 480, 96, 96, PixelFormats.Bgr32, Nothing)

            myScreenImageRect = New Int32Rect(0, 0, CInt(Math.Ceiling(myHumanImage.Width)), CInt(Math.Ceiling(myHumanImage.Height)))
            Kinect.Start()
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Exit Sub
        End Try
    End Sub

    Private Sub RenderScreen(colorFrame As ColorImageFrame, depthFrame As DepthImageFrame)
        If Kinect Is Nothing OrElse depthFrame Is Nothing OrElse colorFrame Is Nothing Then
            Return
        End If
        TextBlock1.Text = String.Empty
        Dim depth As Integer = 0
        Dim depthPixelIndex As Integer
        Dim playerIndex As Integer
        Dim colorPixelIndex As Integer
        Dim colorPoint As ColorImagePoint
        Dim colorStride As Integer = colorFrame.BytesPerPixel * colorFrame.Width
        Dim screenImageStride As Integer = colorFrame.BytesPerPixel * depthFrame.Width

        Dim ImageIndex As Integer = 0
        Dim bytePlayer1 As Byte() = New Byte(1228800 - 1) {}

        depthFrame.CopyPixelDataTo(myDepthPixelData)
        colorFrame.CopyPixelDataTo(myColorPixelData)

        For depthY As Integer = 0 To depthFrame.Height - 1
            Dim depthX As Integer = 0
            While depthX < depthFrame.Width
                depthPixelIndex = depthX + (depthY * depthFrame.Width)
                playerIndex = myDepthPixelData(depthPixelIndex) And DepthImageFrame.PlayerIndexBitmask
                colorPoint = Kinect.MapDepthToColorImagePoint(depthFrame.Format, depthX, depthY, myDepthPixelData(depthPixelIndex), colorFrame.Format)
                colorPixelIndex = (colorPoint.X * colorFrame.BytesPerPixel) + (colorPoint.Y * colorStride)

                If playerIndex <> 0 Then
                    depth = myDepthPixelData(depthPixelIndex) >> DepthImageFrame.PlayerIndexBitmaskWidth '右シフト演算
                    If depth < myDepth Then

                        bytePlayer1(ImageIndex) = myColorPixelData(colorPixelIndex)
                        bytePlayer1(ImageIndex + 1) = myColorPixelData(colorPixelIndex + 1)
                        bytePlayer1(ImageIndex + 2) = myColorPixelData(colorPixelIndex + 2)

                        If rightZ * 1000 < myDepth Then TextBlock1.Text = "これは右手です。"
                        If leftZ * 1000 < myDepth Then TextBlock1.Text = "これは左手です。"
                        If leftZ * 1000 < myDepth AndAlso rightZ * 1000 < myDepth Then TextBlock1.Text = "これは両手です。"
                        If headZ * 1000 < myDepth Then TextBlock1.Text = "これは頭です。"

                    End If
                End If
                depthX = depthX + 1
                ImageIndex = ImageIndex + colorFrame.BytesPerPixel
            End While
        Next
        HumanImage.WritePixels(myScreenImageRect, bytePlayer1, screenImageStride, 0)
    End Sub

    Private Sub kinect_AllFramesReady(sender As Object, e As AllFramesReadyEventArgs)
        Using skeletonFrameData As SkeletonFrame = e.OpenSkeletonFrame()
            If skeletonFrameData Is Nothing = True Then
                Return
            End If

            Dim allSkeletons As Skeleton() = New Skeleton(skeletonFrameData.SkeletonArrayLength - 1) {}
            skeletonFrameData.CopySkeletonDataTo(allSkeletons)
            mySkeleton = (From trackedSkeleton In allSkeletons Where trackedSkeleton.TrackingState = SkeletonTrackingState.Tracked Select trackedSkeleton).FirstOrDefault()
        End Using
        If mySkeleton Is Nothing Then
            Return
        End If
        GetCameraPoint(mySkeleton, e)
    End Sub

    Private Sub GetCameraPoint(ByVal mySkeleton As Skeleton, ByVal e As AllFramesReadyEventArgs)
        Try
            Using depth As DepthImageFrame = e.OpenDepthImageFrame()
                Dim leftDepthPoint As DepthImagePoint = depth.MapFromSkeletonPoint(mySkeleton.Joints(JointType.HandLeft).Position)
                Dim rightDepthPoint As DepthImagePoint = depth.MapFromSkeletonPoint(mySkeleton.Joints(JointType.HandRight).Position)
                Dim headDepthPoint As DepthImagePoint = depth.MapFromSkeletonPoint(mySkeleton.Joints(JointType.Head).Position)

                Dim leftColorPoint As ColorImagePoint = depth.MapToColorImagePoint(leftDepthPoint.X, leftDepthPoint.Y, ColorImageFormat.RgbResolution640x480Fps30)
                Dim rightColorPoint As ColorImagePoint = depth.MapToColorImagePoint(rightDepthPoint.X, rightDepthPoint.Y, ColorImageFormat.RgbResolution640x480Fps30)
                Dim headColorPoint As ColorImagePoint = depth.MapToColorImagePoint(headDepthPoint.X, headDepthPoint.Y, ColorImageFormat.RgbResolution640x480Fps30)

                rightZ = depth.MapToSkeletonPoint(rightColorPoint.X, rightColorPoint.Y).Z
                leftZ = depth.MapToSkeletonPoint(leftColorPoint.X, leftColorPoint.Y).Z
                headZ = depth.MapToSkeletonPoint(headColorPoint.X, headColorPoint.Y).Z
            End Using
        Catch
            Exit Sub
        End Try
    End Sub



    Private Sub MainWindow_Closing(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles Me.Closing
        If Kinect Is Nothing = False Then
            If Kinect.IsRunning = True Then
                Kinect.Stop()
                Kinect.Dispose()
            End If
        End If
    End Sub
End Class


